#define ADD_BLEND_TYPE 0
#define SUB_BLEND_TYPE 1
#define MULTI_BLEND_TYPE 2
#define REPLACE_BLEND_TYPE 3
// #define DISTORTION_BLEND_TYPE 4
#define SCREEN_BLEND_TYPE 5
#define OVERLAY_BLEND_TYPE 6

uniform int blendType;
uniform vec4 blendColor;

varying vec2 v_texCoord;

varying vec4 v_fragmentColor;
varying vec2 blurCoordinates[5];

float blendScreen(float base, float blend) {
    return 1.0 - ((1.0 - base) * (1.0 - blend));
}

float blendOverlay(float base, float blend) {
    return base < 0.5 ? (2.0 * base * blend) : (1.0 - 2.0 * (1.0 - base) * (1.0 - blend));
}

void main()
{
    vec4 sum = vec4(0.0);
    sum += texture2D(CC_Texture0, blurCoordinates[0]) * 0.204164;
    sum += texture2D(CC_Texture0, blurCoordinates[1]) * 0.304005;
    sum += texture2D(CC_Texture0, blurCoordinates[2]) * 0.304005;
    sum += texture2D(CC_Texture0, blurCoordinates[3]) * 0.093913;
    sum += texture2D(CC_Texture0, blurCoordinates[4]) * 0.093913;

    vec3 base = sum.rgb;
    // アルファをブレンドカラーにかけているのは、CC_ENABLE_PREMULTIPLIED_ALPHAが1の前提のコードです。
    vec3 blend = blendColor.rgb * sum.a;
    float blendStr = blendColor.a;
    vec3 reversedBase = base * (1.0 - blendStr);
    if (blendType == ADD_BLEND_TYPE) {
        base = min(base + blend, vec3(1.0)) * blendStr;
    } else if (blendType == SUB_BLEND_TYPE) {
        base = max(base - blend, vec3(0.0)) * blendStr;
    } else if (blendType == MULTI_BLEND_TYPE) {
        base = base * blend * blendStr;
    } else if (blendType == REPLACE_BLEND_TYPE) {
        base = blend * blendStr;
    } else if (blendType == SCREEN_BLEND_TYPE) {
        base = vec3(blendScreen(base.r, blend.r), blendScreen(base.g, blend.g), blendScreen(base.b, blend.b)) * blendStr;
    } else if (blendType == OVERLAY_BLEND_TYPE) {
        base = vec3(blendOverlay(base.r, blend.r), blendOverlay(base.g, blend.g), blendOverlay(base.b, blend.b)) * blendStr;
    }

    sum.rgb = base + reversedBase;
    gl_FragColor = sum * v_fragmentColor;
}

